#!/usr/bin/env node
'use strict';

var _stellarHdWallet = require('stellar-hd-wallet');

var _stellarHdWallet2 = _interopRequireDefault(_stellarHdWallet);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PUBLIC_KEY_LENGTH = 56;

var args = process.argv.slice(2);

// If it's only one argument, default to suffix
if (args.length === 1) {
	args.unshift('-s');
}

var usage = function usage() {
	console.error('Usage:\n\tstellar-vanity-wallet [-p] [-m] [-s] term');
	process.exit(1);
};

// Only alphanumeric seeds are valid
var isValidTerm = function isValidTerm(arg) {
	return (/^[A-Z0-9]+$/.test(arg)
	);
};

// Each argument must have a term to search for
if (args.length % 2 !== 0) {
	usage();
}

// Success must satisfy all match functions
var matchFunctions = [];
var matchWhere = [];

for (var i = 0; i < args.length; i += 2) {
	var flag = args[i];
	var word = args[i + 1].toUpperCase();

	if (!isValidTerm(word)) {
		usage();
	}

	switch (flag) {
		// prefix
		case '-p':
			{
				// Prefixes must begin with A,B,C,D. Anything else is impossible
				if (['A', 'B', 'C', 'D'].indexOf(word[0]) === -1) {
					console.error('Prefix searches must begin with A, B, C or D.');
					process.exit(1);
				}
				matchFunctions.push(function (word) {
					return function (key) {
						return key.indexOf(word) === 1;
					};
				}(word));
				matchWhere.push(word + ' as a prefix');
				break;
			}
		// anywhere
		case '-m':
			{
				matchFunctions.push(function (word) {
					return function (key) {
						return key.indexOf(word) !== -1;
					};
				}(word));
				matchWhere.push(word + ' anywhere');
				break;
			}
		// suffix
		case '-s':
			{
				matchFunctions.push(function (word, suffixIndexMatch) {
					return function (key) {
						return key.indexOf(word) === suffixIndexMatch;
					};
				}(word, PUBLIC_KEY_LENGTH - word.length));
				matchWhere.push(word + ' as a suffix');
				break;
			}
		default:
			{
				usage();
				break;
			}
	}
}

console.log('Looking for ' + matchWhere.join(', '));
console.log('... this may take a while ...');

var found = false;
var mnemonic = void 0;
var wallet = void 0;
var publicKey = void 0;

while (!found) {
	mnemonic = _stellarHdWallet2.default.generateMnemonic();
	wallet = _stellarHdWallet2.default.fromMnemonic(mnemonic);
	publicKey = wallet.getPublicKey(0);
	// console.log('checking', publicKey);

	if (matchFunctions.every(function (fn) {
		return fn(publicKey);
	})) {
		found = true;
	}
}

var secretKey = wallet.getSecret(0);

console.log('Public Key: ' + publicKey);
console.log('Secret Key: ' + secretKey);
console.log('Mnemonic:   ' + mnemonic);